package command

import (
	"context"
	"github.com/open-io/oio-exporter/common"
	_log "github.com/open-io/oio-exporter/common/log"
	"github.com/open-io/oio-exporter/testutils"
	"testing"
)

func TestCommandModule(t *testing.T) {
	t.Parallel()
	_log.LogLevel = _log.INFO
	_ = _log.Setup()

	factory := NewFactory("", "test.yml").(*commandDaemonFactory)
	if err := factory.Reload(); err != nil {
		t.Fatalf("Reload error: %v", err)
	}

	inventory := &common.Inventory{
		Namespace: "TEST",
		Services:  make(common.ServicesMap),
	}
	inventory.Services["service-ok"] = []*common.Service{}

	m := common.MakeMetricData()
	ctx := context.TODO()

	daemonParams := factory.Daemons(inventory)
	for _, params := range daemonParams {
		d := factory.New(params).(*commandDaemon)

		if err := d.Init(ctx); err != nil {
			t.Fatalf("Init(%s) error: %v", d.command.Name, err)
		}

		if err := d.Collect(ctx, m); err != nil {
			t.Fatalf("Collect(%s) error: %v", d.command.Name, err)
		}
		if err := d.Close(); err != nil {
			t.Fatalf("Close(%s) error: %v", d.command.Name, err)
		}
	}

	mdt := testutils.NewMetricDataTest().ForAllMetrics(
		testutils.NewMetricAssertion().HasLabel("command").LabelIs("namespace", "TEST"))

	// default-type
	mdt = mdt.ForMetricsWithLabels(
		testutils.NewMetricAssertion().
			Equals(0),
		"command", "default-type")

	// exec
	mdt = mdt.ForMetricsWithLabels(
		testutils.NewMetricAssertion().
			Equals(0),
		"command", "exec")

	// exec-with-line
	mdt = mdt.ForMetricsWithLabels(
		testutils.NewMetricAssertion().
			Equals(0).
			LabelIs("last_line", "1234"),
		"command", "exec-with-line")

	// shell
	mdt = mdt.ForMetricsWithLabels(
		testutils.NewMetricAssertion().
			Equals(0),
		"command", "shell")

	// shell-with-line
	mdt = mdt.ForMetricsWithLabels(
		testutils.NewMetricAssertion().
			Equals(0).
			LabelIs("last_line", "34"),
		"command", "shell-with-line")

	// return 1
	mdt = mdt.ForMetricsWithLabels(
		testutils.NewMetricAssertion().
			Equals(1),
		"command", "return 1")

	// return 42
	mdt = mdt.ForMetricsWithLabels(
		testutils.NewMetricAssertion().
			Equals(42),
		"command", "return 42")

	// timeout
	mdt = mdt.ForMetricsWithLabels(
		testutils.NewMetricAssertion().
			Equals(110),
		"command", "timeout")

	// no-timeout
	mdt = mdt.ForMetricsWithLabels(
		testutils.NewMetricAssertion().
			Equals(0),
		"command", "no-timeout")

	// service-ok
	mdt = mdt.ForMetricsWithLabels(
		testutils.NewMetricAssertion().
			Equals(0),
		"command", "service-ok")

	// interval
	mdt = mdt.ForMetricsWithLabels(
		testutils.NewMetricAssertion().
			Equals(0).
			LabelIs("interval", "60"),
		"command", "interval")

	// service-ko
	mdt = mdt.ForAllMetrics(testutils.NewMetricAssertion().LabelIsNot("command", "service-ko"))

	mdt.Check(t, m)
}
