package openio

import (
	"context"
	"fmt"
	"strconv"
	"strings"
	"time"

	"github.com/open-io/oio-exporter/common"
	"github.com/open-io/oio-exporter/daemon"
)

type openioRawxDaemonParams struct {
	Namespace     string
	ServiceID     string
	ServiceIP     string
	ServicePort   int
	ServiceVolume string
}

type openioRawxDaemon struct {
	daemon.BaseDaemon

	namespace     string
	serviceID     string
	serviceIP     string
	servicePort   int
	serviceVolume string
	serviceAddr   string
}

func (d *openioRawxDaemon) Interval() time.Duration {
	return confInterval
}

func (d *openioRawxDaemon) Init(ctx context.Context) error {
	d.serviceAddr = fmt.Sprintf("%s:%d", d.serviceIP, d.servicePort)

	return nil
}

func (d *openioRawxDaemon) Collect(ctx context.Context, m *common.MetricData) error {
	serviceAddr := d.serviceAddr

	url := fmt.Sprintf("http://%s/stat", serviceAddr)
	requestCtx, cancel := context.WithTimeout(ctx, confTimeout)
	defer cancel()

	counters, err := common.HTTPGetCsv(requestCtx, url, ' ')
	if err != nil {
		m.IncrError("openio", "rawx")
		return err
	}

	baseLabels := [][]string{
		[]string{"namespace", d.namespace},
		[]string{"service", d.serviceID},
		[]string{"service_type", "rawx"},
		[]string{"service_addr", serviceAddr},
		[]string{"volume", d.serviceVolume},
	}

	for _, cntr := range counters {
		val, err := strconv.ParseFloat(cntr[2], 64)
		if err != nil {
			continue
		}
		if cntr[0] != "counter" {
			continue
		}
		parts := strings.Split(cntr[1], ".")
		if len(parts) == 2 {
			parts = append(parts, "*")
		}
		if len(parts) != 3 {
			continue
		}

		labels := make([][]string, len(baseLabels), len(baseLabels)+1)
		copy(labels, baseLabels)
		if parts[1] == "hits" || parts[1] == "time" {
			if parts[0] == "req" {
				labels = append(labels, []string{"method", parts[2]})
			} else { // "rep"
				labels = append(labels, []string{"code", parts[2]})
			}
		}

		m.Set(&common.Metric{
			Name:   parts[0] + "_" + parts[1],
			Labels: labels,
			Value:  val,
			TTL:    confInterval * 2,
		})
	}

	return nil
}
